--
-- GillesRB410T
-- Specialization for the Gilles RB410T tanker loader
--
-- @author      Benjamin Biot (J&B Modding)
-- @version     v1.0
-- @date          16/03/11
--

GillesRB410T = {};

function GillesRB410T.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Combine, specializations);
end;

function GillesRB410T:load(savegame)
    
    self.threshingScale = 12;
    self.allowThreshingDuringRain = true;
    
    -- UV Scroll for the elevator --
    self.ElevatorUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.ElevatorUVScrollParts.ElevatorUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
         if node ~= nil and speed then
              table.insert(self.ElevatorUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;    
    -- UV Scroll for the beets elevator --
    self.BeetsElevatorUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.BeetsElevatorUVScrollParts.BeetsElevatorUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
         if node ~= nil and speed then
              table.insert(self.BeetsElevatorUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;    
    -- UV Scroll for unloading chain --
    self.UnloadingUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.UnloadingUVScrollParts.UnloadingUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
         if node ~= nil and speed then
              table.insert(self.UnloadingUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;
    -- UV Scroll for discharging plane --
    self.DischargeUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.DischargeUVScrollParts.DischargeUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
         if node ~= nil and speed then
              table.insert(self.DischargeUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;
    
    -- Rotating parts during threshing
    self.numThreshingRotatingParts = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.ThreshingRotatingParts#count"), 0);    
    self.ThreshingRotatingParts = {};
    for i=1, self.numThreshingRotatingParts do
        local RotatingPartsnamei = string.format("vehicle.ThreshingRotatingParts.ThreshingRotatingPart" .. "%d", i);
        self.ThreshingRotatingParts[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, RotatingPartsnamei .. "#index"));
    end;

    -- Variables
    self.cutterAttached = false;

    self.fillLevel = self.grainTankFillLevel;
    self.currentFillType = Fillable.FILLTYPE_SUGARBEET;
end;

function GillesRB410T:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

end;

function GillesRB410T:getSaveAttributesAndNodes(nodeIdent)

end;

function GillesRB410T:delete()

end;

function GillesRB410T:readStream(streamId, connection)

end;

function GillesRB410T:writeStream(streamId, connection)

end;

function GillesRB410T:readUpdateStream(streamId, timestamp, connection)
end;

function GillesRB410T:writeUpdateStream(streamId, connection, dirtyMask)
end;

function GillesRB410T:mouseEvent(posX, posY, isDown, isUp, button)
end;

function GillesRB410T:keyEvent(unicode, sym, modifier, isDown)
end;

function GillesRB410T:update(dt)
    if self:getIsActive() then
        for _, part in pairs(self.movingParts) do
            part.isDirty = true;
        end;    

        -- Handling UV Scroll --
        for _, part in pairs(self.ElevatorUVScrollParts) do
            if self.isThreshing then
                setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
            else
                setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
            end;
        end;
        for _, part in pairs(self.BeetsElevatorUVScrollParts) do
            if self.isThreshing then
                setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
            else
                setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
            end;
        end;
        -- Threshing rotating parts --
        if self.isThreshing then
            for i=1, self.numThreshingRotatingParts-1 do
                rotate(self.ThreshingRotatingParts[i], 0, 0, -0.02 * dt);
            end;
            for i= self.numThreshingRotatingParts, self.numThreshingRotatingParts do
                rotate(self.ThreshingRotatingParts[i], -0.01 * dt, 0, 0);
            end;
        end;
        -- Unloading UVs --
        
        for _, part in pairs(self.UnloadingUVScrollParts) do
            if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
                setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
            else
                setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
            end;
        end;
        
        for _, part in pairs(self.DischargeUVScrollParts) do
                if (self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN) and self.fillLevel > 0 then
                    setVisibility(part.node, true);
                    setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);    
                    Utils.setEmittingState(self.dischargeParticleSystems[self.currentFillType], true);
                else
                    setVisibility(part.node, false);
                    setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
                    Utils.setEmittingState(self.dischargeParticleSystems[self.currentFillType], false);
                end;
        end;
        
        -- Display the moving "beets faces" 
        if self.strawPSenabled then
            for _, part in pairs(self.BeetsElevatorUVScrollParts) do
                setVisibility(part.node, true);
            end;
        else
            for _, part in pairs(self.BeetsElevatorUVScrollParts) do
                setVisibility(part.node, false);
            end;
        end;
    end;
    
    -- From auto attach mod : attaching the cutter when it is in range
    -- THIS SCRIPT IS NOT MP COMPATIBLE
    --[[if not self.cutterAttached then
        local nearestDistance = 1.5;
        if not self.attacherJoints[1].jointIndex ~= 0 then
            local px, py, pz = getWorldTranslation(self.attacherJoints[1].jointTransform);
            for k,attachable in pairs(g_currentMission.attachables) do
                local attacherJoint = attachable.attacherJoint;
                local vx, vy, vz = getWorldTranslation(attacherJoint.node);
                local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
                if distance < nearestDistance then
                    self:attachImplement(attachable, 1);
                end;
            end;
        end;
    end;
    self.cutterAttached = true;]]


    
end;

function GillesRB410T:updateTick(dt)    
    if self:getIsActive() then
        self.currentFillType = Fillable.FILLTYPE_SUGARBEET;
        -- tanker is closed. Update trailer capacity according to tank capacity
        if self.tipState == Trailer.TIPSTATE_CLOSED then
            self.fillLevel = self.grainTankFillLevel;
        end;
        -- tanker is closing or opening or opened. Update tank capacity and grainPlane position according to trailer capacity.
        -- Manage unloading animation
        if self.tipState == Trailer.TIPSTATE_CLOSING or self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
            self.grainTankFillLevel = self.fillLevel;
            local fruitType = FruitUtil.FRUITTYPE_SUGARBEET;
            local fruitTypeName = FruitUtil.fruitIndexToDesc[fruitType].name;
            local grainPlane = self.grainTankPlanes[fruitTypeName];
            local t = self.grainTankFillLevel/self.grainTankCapacity
            for _, node in ipairs(grainPlane.nodes) do
              local x,y,z, rx,ry,rz, sx,sy,sz = node.animCurve:get(t);

              setTranslation(node.node, x, y, z);
              setRotation(node.node, rx, ry, rz);
              setScale(node.node, sx, sy, sz);
              setVisibility(node.node, self.grainTankFillLevel > 0);
            end;    
            
        end;
    end;
end;

function GillesRB410T:draw()

end;

function GillesRB410T:onLeave()
    for _, part in pairs(self.ElevatorUVScrollParts) do
        setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
    end;
end;





