--
-- TurboCleanCutter
-- Specialization for the Gilles RB410T's cutter
--
-- @author      Benjamin Biot (J&B Modding)
-- @version     v1.0
-- @date          31/03/11
--

TurboCleanCutter = {};

function TurboCleanCutter.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Cutter, specializations);
end;

function TurboCleanCutter:load(savegame)
    self.setTransport= SpecializationUtil.callSpecializationsFunction("setTransport");    
    -- Animations --
    self.TransportAnimation = getXMLString(xmlFile, "vehicle.Transport#animationName");
    self.Transport = true;
    -- cutter wheels --
    self.cutterWheels = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cutterWheels#index"));
    -- ground detector
    self.groundDetector = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundIndex#index"));

end;

function TurboCleanCutter:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

end;

function TurboCleanCutter:getSaveAttributesAndNodes(nodeIdent)

end;

function TurboCleanCutter:delete()

end;

function TurboCleanCutter:readStream(streamId, connection)
    self:setTransport(streamReadBool(streamId), true);
end;

function TurboCleanCutter:writeStream(streamId, connection)
    streamWriteBool(streamId, self.Transport);
end;

function TurboCleanCutter:readUpdateStream(streamId, timestamp, connection)
end;

function TurboCleanCutter:writeUpdateStream(streamId, connection, dirtyMask)
end;

function TurboCleanCutter:mouseEvent(posX, posY, isDown, isUp, button)
end;

function TurboCleanCutter:keyEvent(unicode, sym, modifier, isDown)
end;

function TurboCleanCutter:update(dt)        
    if self:getIsActiveForInput() then
        -- Trigger the transport animation --
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
            self:setTransport(not self.Transport);
        end;
    end;
    -- rotate cutter wheels
    if self:getIsActive() then
        local x,y,z = getWorldTranslation(self.groundDetector);
        local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
        if self.Transport and terrainHeight+0.1 >= y then
            rotate(self.cutterWheels, -2 * self.lastSpeedReal * self.movingDirection * dt, 0, 0);
        end;
    end;
end;

function TurboCleanCutter:updateTick(dt)    
    if self.isServer then
        self.lastArea = 0;
    end;
    if self.reelStarted and FruitUtil.FRUITTYPE_SUGARBEET ~= nil and self.movingDirection < 0 then
        if self.isServer then
            local x,y,z = getWorldTranslation(self.groundDetector);
            local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
            if self.Transport and terrainHeight+0.2 >= y then
                self.currentFruitType = FruitUtil.FRUITTYPE_SUGARBEET;
                self.currentInputFruitType = self.currentFruitType;
                --local realArea = 0;
                local cuttingAreasSend = {};
                for k,area in pairs(self.cuttingAreas) do
                    if self:getIsAreaActive(area) then
                        local x,y,z = getWorldTranslation(area.start);
                        local x1,y1,z1 = getWorldTranslation(area.width);
                        local x2,y2,z2 = getWorldTranslation(area.height);
                        table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
                    end;
                end;
                if (table.getn(cuttingAreasSend) > 0) then
                    local lastArea, realArea = SugarBeetCutterAreaEvent.runLocally(cuttingAreasSend, self.currentInputFruitType);
                    self.lastArea = lastArea;
                    self.lastAreaBiggerZero = (self.lastArea > 0);
                    if self.lastArea > 0 then
                        g_server:broadcastEvent(SugarBeetCutterAreaEvent:new(cuttingAreasSend, self.currentInputFruitType));
                    end;            
                    if self.lastAreaBiggerZero ~= self.lastAreaBiggerZeroSent then
                        self:raiseDirtyFlags(self.cutterGroundFlag);
                        self.lastAreaBiggerZeroSent = self.lastAreaBiggerZero;
                    end;
                    local pixelToSqm = g_currentMission:getFruitPixelsToSqm(); -- 4096px are mapped to 2048m
                    local sqm = realArea*pixelToSqm;
                    local ha = sqm/10000;
                
                    g_currentMission.missionStats.hectaresThreshedTotal = g_currentMission.missionStats.hectaresThreshedTotal + ha;
                    g_currentMission.missionStats.hectaresThreshedSession = g_currentMission.missionStats.hectaresThreshedSession + ha;
                end;

                g_currentMission.missionStats.threshingDurationTotal = g_currentMission.missionStats.threshingDurationTotal + dt/(1000*60);
                g_currentMission.missionStats.threshingDurationSession= g_currentMission.missionStats.threshingDurationSession + dt/(1000*60);
            end;
        end;
    end;

                 
end;

function TurboCleanCutter:draw()
    if self.Transport then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("TRANSPORT_OFF"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA3);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("TRANSPORT_ON"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA3);
    end;
end;

function TurboCleanCutter:onLeave()
end;

function TurboCleanCutter:onDetach()
end;

function TurboCleanCutter:onAttach()
end;

function TurboCleanCutter:setTransport(isTransport,noEventSend)
    SetTransportEvent.sendEvent(self, isTransport, noEventSend);
    self.Transport = isTransport;
    -- Play transport animation --
    if self.Transport then
        if self.TransportAnimation ~= nil and self.playAnimation ~= nil then
            self:playAnimation(self.TransportAnimation, -1, nil, true);
            self.Transport = true;
        end;
    else
        if self.TransportAnimation ~= nil and self.playAnimation ~= nil then
            self:playAnimation(self.TransportAnimation, 1, nil, true);
            self.Transport = false;
        end;
    end;
end;
